@@ -1,7 +1,7 @@
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
 #
 #   YADT - an Augmented Deployment Tool
-#   Copyright (C) 2010-2013  Immobilien Scout GmbH
+#   Copyright (C) 2010-2014  Immobilien Scout GmbH
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -124,7 +124,11 @@ class ActionManager(object):
         component = self.components[uri]
         deferred = None
 
-        self.logger.info('/' + '/'.join(path))
+        action_to_execute = '/' + '/'.join(path)
+        if self.dryrun:
+            self.logger.info(action_to_execute)
+        else:
+            self.logger.debug(action_to_execute)
         if cmd == yadtshell.settings.FINISH:
             if self.finish_fun:
                 self.finish_fun(action)
@@ -274,9 +278,6 @@ class ActionManager(object):
         p.target_state = target_state
         p.state = yadtshell.settings.UNKNOWN
 
-        # if self.pi:
-            # self.pi.observables.append(p)
-        p.deferred = defer.Deferred()
         cmdline = shlex.split(cmdline.encode('ascii'))
         self.logger.debug('cmd: %s' % cmdline)
         reactor.spawnProcess(p, cmdline[0], cmdline, None)
@@ -339,7 +340,7 @@ class ActionManager(object):
             queue.append(yadtshell.ActionManager.Task(
                 fun=self.handle, action=action, path=this_path))
         plan.nr_workers = min(plan.nr_workers, len(queue))
-        self.logger.info('%s : %s' % (plan_name, plan.meta_info()))
+        self.logger.debug('%s : %s' % (plan_name, plan.meta_info()))
 
         pool = yadtshell.defer.DeferredPool(
             plan_name,
@@ -376,10 +377,8 @@ class ActionManager(object):
             f.close()
         except IOError, e:
             self.logger.warning(str(e))
-            # return defer.succeed(None)
             deferred = defer.Deferred()
             reactor.callLater(0, deferred.errback, e)
-                              # TODO mhhh: good practice?
             return deferred
 
         if not action_plan:
@@ -393,10 +392,8 @@ class ActionManager(object):
         for service in [s for s in self.components.values() if s.type == yadtshell.settings.SERVICE]:
             service.state = yadtshell.settings.UNKNOWN
         for host in [h for h in self.components.values() if isinstance(h, yadtshell.components.Host)]:
-            setattr(
-                host, 'state', yadtshell.settings.UNKNOWN)
-            setattr(
-                host, yadtshell.constants.PROBED, yadtshell.settings.UNKNOWN)
+            host.state = yadtshell.settings.UNKNOWN
+            host.probed = yadtshell.settings.UNKNOWN
 
         if dryrun:
             log_plan_fun = self.logger.info
@@ -426,11 +423,6 @@ class ActionManager(object):
                 pi.finish()
             return result
 
-        def filter_dangerous_actions(actions):
-            def is_a_dangerous_action(action):
-                return action.cmd in ['update'] and action.kwargs.get(yadtshell.constants.REBOOT_REQUIRED, False)
-            return filter(is_a_dangerous_action, actions)
-
         if _user_should_acknowledge_plan(dryrun, flavor, forcedyes):
             dangerous_actions = filter_dangerous_actions(action_plan.list_actions)
             if dangerous_actions:
@@ -465,6 +457,12 @@ class ActionManager(object):
         return deferred
 
 
+def filter_dangerous_actions(actions):
+    def is_a_dangerous_action(action):
+        return action.cmd in ['update'] and action.kwargs.get(yadtshell.constants.REBOOT_REQUIRED, False)
+    return filter(is_a_dangerous_action, actions)
+
+
 def _user_should_acknowledge_plan(dryrun, flavor, forcedyes):
     if dryrun:
         return False
