@@ -1,7 +1,7 @@
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
 #
 #   YADT - an Augmented Deployment Tool
-#   Copyright (C) 2010-2013  Immobilien Scout GmbH
+#   Copyright (C) 2010-2014  Immobilien Scout GmbH
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -25,9 +25,11 @@ import sys
 import yaml
 import shlex
 
-from twisted.internet import defer, reactor, task
+from twisted.internet import reactor, task
 
 from yadtshell.util import calculate_max_tries_for_interval_and_delay
+from yadtshell.helper import get_user_info
+from yadtshell.twisted import YadtProcessProtocol
 import yadtshell
 
 
@@ -35,31 +37,35 @@ logger = logging.getLogger('components')
 
 
 class Component(object):
+    """Abstract superclass for Host, Service, Artefacts, and some special cases thereof.
 
-    def __init__(self, t, host, name=None, version=None):
+    Note that the `.host` attribute is always a string, not a Host instance.
+    TODO(rwill): rename it to `.hostname` or change it to a Host instance (as passed into constructor anyway).
+    """
+
+    def __init__(self, t, host, name):
+        """Makes a component.
+
+        - `t` is one of the component types in yadtshell.settings
+        - `host` must be a true Host instance so we can set `fqdn` properly.
+        - `name` is a plain string and doesn't contain a version.
+
+        Note the constructor is written such that subclasses can call it first
+        and then overwrite any values if so desired. The pattern here is "initialize
+        with a default value and overwrite if needed".
+        """
         self.type = t
-        if type(host) in [str, unicode]:
-            self.host = host
-        else:
-            self.host = host.host
-            self.fqdn = host.fqdn
-        self.host_uri = yadtshell.uri.create(
-            yadtshell.settings.HOST, self.host)
-        self.name = getattr(self, 'name', name)
-        if self.name is not None:
-            self.name = self.name.rstrip('/').split('/', 1)[0]
-        self.version = version
-        if self.version is not None:
-            self.version = str(self.version)
-        self.uri = yadtshell.uri.create(
-            self.type, self.host, self.name, self.version)
-        self.version = yadtshell.uri.parse(self.uri)['version']
-        self.name_and_version = yadtshell.uri.as_source_file(self.uri)
+        self.name = name
+        self.host = host.name
+        self.fqdn = host.fqdn
+        self.uri = yadtshell.uri.create(self.type, self.host, self.name)
+
+        self.host_uri = yadtshell.uri.create(yadtshell.settings.HOST, self.host)  # TODO(rwill): seems like dead code
+
         self.state = yadtshell.settings.UNKNOWN
-        self.revision = yadtshell.settings.EMPTY
-        self.needs = getattr(self, 'needs', set())
+        self.needs = set()
         self.needed_by = set()
-        self.config_prefix = yadtshell.settings.TARGET_SETTINGS['name']
+        # self.config_prefix = yadtshell.settings.TARGET_SETTINGS['name']  # TODO(rwill): dead code?
 
     def is_touched_also(self, other):
         return True
@@ -104,9 +110,9 @@ class Component(object):
         service = self.name
         remotecall_script = '/usr/bin/yadt-remotecall'
         log_file = self.create_remote_log_filename(tag=tag)
-        owner = yadtshell.util.get_locking_user_info()['owner']
-        is_force = {False: '', True: ' --force'}[force]
-        complete_cmd = '%(ssh_cmd)s %(host)s WHO="%(owner)s" YADT_LOG_FILE="%(log_file)s" "yadt-command %(cmd)s%(is_force)s" ' % locals()
+        owner = get_user_info()['owner']
+        force_flag = {False: '', True: ' --force'}[force]
+        complete_cmd = '%(ssh_cmd)s %(host)s WHO="%(owner)s" YADT_LOG_FILE="%(log_file)s" "yadt-command %(cmd)s%(force_flag)s" ' % locals()
         return complete_cmd
 
     def local_call(self, cmd, tag=None, guard=True, force=False, no_subprocess=True):
@@ -120,9 +126,9 @@ class Component(object):
         if no_subprocess:
             return cmds
         if guard:
-            sp = self.remote_call(
-                ": #check service callable", tag='check_service_callable',
-                force=force)
+            sp = self.remote_call(": #check service callable",
+                                  tag='check_service_callable',
+                                  force=force)
             returncode = yadtshell.util.log_subprocess(sp)
             if returncode != 0:
                 return returncode
@@ -151,11 +157,13 @@ class Component(object):
 
 
 class MissingComponent(Component):
+    """TODO(rwill): What is the usecase for this? Add tests or remove it.
+    """
 
     def __init__(self, s):
         parts = yadtshell.uri.parse(s)
-        Component.__init__(self, parts['type'], parts[
-                           'host'], parts['name'], parts['version'])
+        Component.__init__(self, parts['type'], Host(parts['host']), parts['name'])
+        self.version = parts['version']
         self.state = yadtshell.settings.MISSING
 
 
@@ -167,7 +175,7 @@ class ComponentDict(dict):
 
     def _key_(self, key):
         try:
-            return self._key_(key.uri)
+            return self._key_(key.uri)  # TODO(rwill): looks like the intention here is to return key.uri ??
         except AttributeError:
             return key
 
@@ -180,12 +188,11 @@ class ComponentDict(dict):
     def get(self, key, default=None):
         if self._key_(key) not in self and self._add_when_missing_:
             logger.debug('missing' + key)
-            self[self._key_(key)] = MissingComponent(key)
+            self[self._key_(key)] = MissingComponent(key)  # (rwill) why no _key_ on RHS?
         return dict.get(self, self._key_(key), default)
 
     def __setitem__(self, key, value):
-        # logger.debug('adding ' + self._key_(key) + ' ' + getattr(value,
-        # 'state', ''))
+        # logger.debug('adding ' + self._key_(key) + ' ' + getattr(value, 'state', ''))
         return dict.__setitem__(self, self._key_(key), value)
 
 
@@ -206,8 +213,7 @@ class ComponentSet(set):
         logger.debug('adding ' + key)
         if key not in self.components and check:
             logger.warning('key %(key)s not found, ignoring' % locals())
-            # logger.warning('known keys: ' + ',
-            # '.join(self.components.keys()))
+            # logger.warning('known keys: ' + ', '.join(self.components.keys()))
             return None
         return self._set.add(key)
 
@@ -228,9 +234,29 @@ class ComponentSet(set):
         return self._key_(item) in self._set
 
 
-class Host(Component):
+class AbstractHost(Component):
+    def __init__(self, fqdn):
+        # we need to set those values first, because Component.__init__ requires them on its `host` argument ;)
+        self.fqdn = fqdn
+        self.hostname = fqdn.split('.')[0]
+        Component.__init__(self, yadtshell.settings.HOST, self, self.hostname)
+        self.uri = yadtshell.uri.create(yadtshell.settings.HOST, self.name)
+
+        self.current_artefacts = []
+        self.next_artefacts = []
+        self.services = {}
+
+
+class Host(AbstractHost):
+    """Note: `Host.name`, `Host.host` and `Host.hostname` are all the same value.
+    We need `.host` and `.name` because they are part of Component, but when
+    `Component.host` is renamed to Component.hostname, we will at least have
+    one less redundant variable.
+    """
+
+    def __init__(self, fqdn):
+        AbstractHost.__init__(self, fqdn)
 
-    def __init__(self, name):
         self.lockstate = None
         self.is_locked = None
         self.is_locked_by_other = None
@@ -238,7 +264,24 @@ class Host(Component):
         self.ssh_poll_max_seconds = yadtshell.constants.SSH_POLL_MAX_SECONDS_DEFAULT
         self.reboot_required_to_activate_latest_kernel = False
         self.reboot_required_after_next_update = False
-        Component.__init__(self, yadtshell.settings.HOST, name)
+
+        self.logger = logging.getLogger(self.uri)
+
+    def set_attrs_from_data(self, data):
+        for key, value in data.iteritems():
+            if key == "hostname" and value != self.hostname:
+                self.logger.warning("Hostname %(hostname)s doesn't match FQDN %(fqdn)." % data)
+            setattr(self, key, value)
+        self.convert_obsolete_services(self.services)
+        self.state = ['update_needed', 'uptodate'][not self.next_artefacts]
+        self.loc_type = yadtshell.util.determine_loc_type(self.hostname)
+        self.update_attributes_after_status()
+
+    def convert_obsolete_services(self, old_services):
+        if len(old_services) > 0 and type(old_services) is list:
+            self.services = dict()
+            for entry in old_services:
+                self.services.update(entry)
 
     @property
     def reboot_required(self):
@@ -251,55 +294,46 @@ class Host(Component):
         next_artefacts = [uri.replace('/', '-', 1)
                           for uri in self.next_artefacts]
         if not reboot_required:
-            return self.remote_call(
-                'yadt-host-update %s' % ' '.join(next_artefacts), '%s_%s' %
-                (self.hostname, yadtshell.settings.UPDATE))
+            return self.remote_call('yadt-host-update %s' % ' '.join(next_artefacts),
+                                    '%s_%s' % (self.hostname, yadtshell.settings.UPDATE))
 
         update_and_reboot_command = self.remote_call(
-            'yadt-host-update -r %s' % ' '.join(next_artefacts), '%s_%s' %
-            (self.hostname, yadtshell.settings.UPDATE))
-        p = yadtshell.twisted.YadtProcessProtocol(self,
-                                                  update_and_reboot_command,
-                                                  out_log_level=logging.INFO)
+            'yadt-host-update -r %s' % ' '.join(next_artefacts),
+            '%s_%s' % (self.hostname, yadtshell.settings.UPDATE))
+        p = YadtProcessProtocol(self, update_and_reboot_command, out_log_level=logging.INFO)
         p.target_state = yadtshell.settings.UPTODATE
         p.state = yadtshell.settings.UNKNOWN
 
-        p.deferred = defer.Deferred()
-
         def handle_rebooting_machine(failure, ssh_poll_max_seconds):
             if failure.value.exitCode == 152:
                 raise yadtshell.actions.ActionException(
                     'Timed out while waiting for %s to reboot' % self.uri)
             elif failure.value.exitCode == 255:
                 logger.info("%s: rebooting now" % self.uri)
-                return poll_rebooting_machine(ssh_poll_max_seconds=ssh_poll_max_seconds)
+                return poll_rebooting_machine()
             return failure
 
-        def poll_rebooting_machine(count=1, ssh_poll_max_seconds=yadtshell.constants.SSH_POLL_MAX_SECONDS_DEFAULT):
-            max_tries = calculate_max_tries_for_interval_and_delay(interval=ssh_poll_max_seconds,
+        def poll_rebooting_machine(count=1):
+            max_tries = calculate_max_tries_for_interval_and_delay(interval=self.ssh_poll_max_seconds,
                                                                    delay=yadtshell.constants.SSH_POLL_DELAY)
             logger.info("%s: polling for ssh connect, try %i of %i" %
                         (self.uri, count, max_tries))
-            poll_command = self.remote_call(
-                'uptime', '%s_poll' % self.hostname)
-            poll_protocol = yadtshell.twisted.YadtProcessProtocol(
-                self, poll_command, out_log_level=logging.INFO)
+            poll_command = self.remote_call('uptime', '%s_poll' % self.hostname)
+            poll_protocol = YadtProcessProtocol(self, poll_command, out_log_level=logging.INFO)
             poll_protocol.ssh_poll_count = count
-            poll_protocol.deferred = defer.Deferred()
-            if (count * yadtshell.constants.SSH_POLL_DELAY) < ssh_poll_max_seconds:
-                poll_protocol.deferred.addErrback(lambda x:
-                                                  task.deferLater(reactor,
-                                                                  yadtshell.constants.SSH_POLL_DELAY,
-                                                                  poll_rebooting_machine,
-                                                                  count + 1,
-                                                                  ssh_poll_max_seconds=ssh_poll_max_seconds))
+            if (count * yadtshell.constants.SSH_POLL_DELAY) < self.ssh_poll_max_seconds:
+                poll_protocol.deferred.addErrback(
+                    lambda x: task.deferLater(reactor,
+                                              yadtshell.constants.SSH_POLL_DELAY,
+                                              poll_rebooting_machine,
+                                              count + 1)
+                )
             cmdline = shlex.split(poll_protocol.cmd)
             reactor.spawnProcess(poll_protocol, cmdline[0], cmdline, None)
 
             return poll_protocol.deferred
 
-        p.deferred.addErrback(
-            handle_rebooting_machine, self.ssh_poll_max_seconds)
+        p.deferred.addErrback(handle_rebooting_machine, self.ssh_poll_max_seconds)
 
         def display_reboot_info(protocol):
             if hasattr(protocol, 'ssh_poll_count'):
@@ -344,7 +378,7 @@ class Host(Component):
             return message.replace("'", "").replace('"', '')
         if not message:
             raise ValueError('the "message" parameter is mandatory')
-        lockinfo = yadtshell.util.get_locking_user_info()
+        lockinfo = get_user_info()
         lockinfo["message"] = message
         lockinfo["force"] = force
         return self.remote_call(
@@ -356,27 +390,23 @@ class Host(Component):
         return self.remote_call('yadt-host-unlock', "unlock_host", force)
 
     def update_attributes_after_status(self):
-        self.is_locked = not self.lockstate is None
+        self.is_locked = self.lockstate is not None
 
-        lockinfo = yadtshell.util.get_locking_user_info()
+        lockinfo = get_user_info()
         lock_owner = None
         if self.lockstate:
             lock_owner = self.lockstate.get("owner")
-        self.is_locked_by_me = self.is_locked and lock_owner and lock_owner == lockinfo[
-            "owner"]
-
+        self.is_locked_by_me = self.is_locked and lock_owner and lock_owner == lockinfo["owner"]
         self.is_locked_by_other = self.is_locked and not self.is_locked_by_me
 
         logger.debug("is_locked=" + repr(self.is_locked) + ", is_locked_by_me=" + repr(
             self.is_locked_by_me) + ", is_locked_by_other=" + repr(self.is_locked_by_other))
 
 
-class UnreachableHost(Component):
+class UnreachableHost(AbstractHost):
 
-    def __init__(self, name):
-        self.fqdn = name
-        self.hostname = self.fqdn.split('.')[0]
-        Component.__init__(self, yadtshell.settings.HOST, name)
+    def __init__(self, fqdn):
+        AbstractHost.__init__(self, fqdn)
 
     def is_reachable(self):
         return False
@@ -384,10 +414,6 @@ class UnreachableHost(Component):
     def is_unknown(self):
         return True
 
-    @property
-    def next_artefacts(self):
-        return {}
-
     @property
     def is_locked_by_other(self):
         return False
@@ -398,11 +424,17 @@ class UnreachableHost(Component):
 
 
 class Artefact(Component):
+    """`version` is numeric
+    `revision` is either 'next' or 'current'. (Use constants in yadtshell.settings!)
+    """
 
-    def __init__(self, host, name, version=None):
-        Component.__init__(
-            self, yadtshell.settings.ARTEFACT, host, name, version)
-        # self.needs.add(uri.create(settings.HOST, host.host))
+    def __init__(self, host, name, version, revision=yadtshell.settings.CURRENT):
+        Component.__init__(self, yadtshell.settings.ARTEFACT, host, name)
+        self.version = version
+        self.revision = revision
+        self.state = yadtshell.settings.INSTALLED  # only needed for yadtshell.util.is_up()
+        self.uri = yadtshell.uri.create(yadtshell.settings.ARTEFACT, host.name, name, version)
+        self.revision_uri = yadtshell.uri.create(yadtshell.settings.ARTEFACT, host.name, name, revision)
 
     def updateartefact(self):
         return self.remote_call('yadt-artefact-update %s' % self.name,
@@ -416,7 +448,7 @@ class Service(Component):
 
         self.needs_services = []
         self.needs_artefacts = []
-        self.needs = set()
+        self.needs.add(host.uri)
 
         for k in settings:
             setattr(self, k, settings[k])
@@ -434,8 +466,9 @@ class Service(Component):
                 self.needs.add(yadtshell.uri.create(
                     yadtshell.settings.SERVICE, host.host, n % locals()))
         for n in self.needs_artefacts:
-            self.needs.add(yadtshell.uri.create(yadtshell.settings.ARTEFACT, host.host, n % locals() +
-                                                "/" + yadtshell.settings.CURRENT))
+            self.needs.add(yadtshell.uri.create(yadtshell.settings.ARTEFACT,
+                                                host.host,
+                                                n % locals() + "/" + yadtshell.settings.CURRENT))
         # self.needs.add(uri.create(yadtshell.settings.HOST, host.host))
 
         self.state = yadtshell.settings.STATE_DESCRIPTIONS.get(
@@ -466,7 +499,7 @@ class Service(Component):
             raise ValueError('the "message" parameter is mandatory')
         tag = "ignore_%s" % self.name
         force = kwargs.get('force', False)
-        return self.remote_call('yadt-service-ignore %s "%s"' % (self.name, message), tag, force)
+        return self.remote_call('yadt-service-ignore %s \'%s\'' % (self.name, message), tag, force)
 
     def unignore(self, **kwargs):
         tag = "unignore_%s" % self.name
@@ -491,8 +524,7 @@ def do(args, opts):
     for component_name in component_names:
         component = components.get(component_name, None)
         if not component:
-            component = components[
-                yadtshell.uri.change_version(component_name, 'current')]
+            component = components[yadtshell.uri.change_version(component_name, 'current')]
         fun = getattr(component, cmd, None)
         import inspect
         if inspect.ismethod(fun):
@@ -501,19 +533,16 @@ def do(args, opts):
             except TypeError:
                 sp = fun()
         else:
-            logger.error(
-                '"%(cmd)s" is not defined for %(component_name)s, aborting' % locals())
+            logger.error('"%(cmd)s" is not defined for %(component_name)s, aborting' % locals())
             sys.exit(2)
         logger.debug('%(cmd)sing %(component_name)s' % locals())
         try:
             logger.debug('executing fun ' + str(fun))
             if isinstance(sp, subprocess.Popen):
-                exit_code = yadtshell.util.log_subprocess(
-                    sp, stdout_level=logging.INFO)
+                exit_code = yadtshell.util.log_subprocess(sp, stdout_level=logging.INFO)
             else:
                 exit_code = sp
             logger.debug('exit code %(exit_code)s' % locals())
         except AttributeError, ae:
-            logger.warning(
-                'problem while executing %(cmd)s on %(component_name)s' % locals())
+            logger.warning('problem while executing %(cmd)s on %(component_name)s' % locals())
             logger.exception(ae)
