@@ -1,7 +1,7 @@
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
 #
 #   YADT - an Augmented Deployment Tool
-#   Copyright (C) 2010-2013  Immobilien Scout GmbH
+#   Copyright (C) 2010-2014  Immobilien Scout GmbH
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 import logging
 import re
+import sys
 
 import util
 
@@ -31,7 +32,13 @@ def dump(args=[], mode='all', attribute=None, filter=None, **kwargs):
     if kwargs.get('show_current_artefacts'):
         args = ['host://']
         attribute = 'handled_artefacts'
-    components = util.restore_current_state()
+    try:
+        components = util.restore_current_state()
+    except IOError:
+        logger.critical("cannot restore the current state")
+        logger.info("call 'yadtshell status' first")
+        sys.exit(1)
+
     result = set()
     for uri in components.keys():
         if len(args) > 0:
