@@ -1,7 +1,7 @@
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
 #
 #   YADT - an Augmented Deployment Tool
-#   Copyright (C) 2010-2013  Immobilien Scout GmbH
+#   Copyright (C) 2010-2014  Immobilien Scout GmbH
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -17,8 +17,9 @@
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 from __future__ import print_function
 import logging
-import time
+import sys
 from subprocess import Popen, PIPE
+import time
 
 import hostexpand
 import yadtshell
@@ -53,10 +54,26 @@ def _show_host_locking_or_unreachable(host):
                   (host.host, lock_owner, "reason", reason)))
 
 
+def _show_ignored_services(services):
+    ignored_services = False
+    for service in services:
+        if hasattr(service, 'ignored'):
+            ignored_services = True
+            print(render_yellow('\n%20s is ignored\n%10s' % (service, service.ignored.get('message', 'no message'))))
+
+    if ignored_services:
+        print()  # separate ignored services from locked hosts
+
+
 def info(logLevel=None, full=False, components=None, **kwargs):
     if not components:
         logger.debug("loading current state")
-        components = yadtshell.util.restore_current_state()
+        try:
+            components = yadtshell.util.restore_current_state()
+        except IOError:
+            logger.critical("cannot restore current state")
+            logger.info("call 'yadtshell status' first")
+            sys.exit(1)
 
     result = []
     for component in components.values():
@@ -69,6 +86,11 @@ def info(logLevel=None, full=False, components=None, **kwargs):
     print()
     print('target status')
 
+    services = [component for component in components.values()
+                if isinstance(component, yadtshell.components.Service)]
+
+    _show_ignored_services(services)
+
     hosts = sorted(
         [c for c in components.values() if c.type == yadtshell.settings.HOST], key=lambda h: h.uri)
     for host in hosts:
@@ -96,8 +118,7 @@ def info(logLevel=None, full=False, components=None, **kwargs):
         print(render_red('\nconfig problem: missing %s\n' %
               missing_component.uri))
 
-    for service in [component for component in components.values()
-                    if isinstance(component, yadtshell.components.Service)]:
+    for service in services:
         if getattr(service, 'service_artefact_problem', None):
             print(
                 render_red('problem with %(uri)s\n\t%(service_artefact)s: %(service_artefact_problem)s\n\t-> no artefact dependencies available!\n' % vars(service)))
@@ -122,7 +143,9 @@ def _render_updates_based_on_name_schema(components, host, full):
 
         host_artefacts = {}
         for current_artefact in [
-            c for c in components.values() if c.host == host.hostname and c.revision == yadtshell.settings.CURRENT
+            c for c in components.values() if (c.type == yadtshell.settings.ARTEFACT
+                                               and c.host == host.hostname
+                                               and c.revision == yadtshell.settings.CURRENT)
         ]:
             artefact = host_artefacts.setdefault(current_artefact.name, {})
             artefact[yadtshell.settings.CURRENT] = current_artefact
@@ -199,8 +222,10 @@ def calculate_info_view_settings():
     except:
         cols = 80
 
+    logger.debug("expanding hosts")
     he = hostexpand.HostExpander.HostExpander()
     max_row_length = max([len(he.expand(hosts)) for hosts in original_hosts])
+    logger.debug("expanded hosts")
 
     width = '1col'
 
@@ -251,7 +276,7 @@ def _render_services_matrix(components, hosts, info_view_settings, enable_legend
                 service = servicedef.keys()[0]
             except:
                 service = servicedef
-            if not service in services:
+            if service not in services:
                 rank = components[yadtshell.uri.create(
                     yadtshell.settings.SERVICE, host.hostname, service)].dependency_score
                 services.append((rank, service))
